@@ -315,9 +315,9 @@ md5_sum_update (Md5sum       *md5,
       guchar *p = (guchar *) md5->data + bit;
 
       bit = MD5_DATASIZE - bit;
-      if (length < MD5_DATASIZE)
+      if (length < bit)
         {
-          memcpy (p, data, bit);
+          memcpy (p, data, length);
           return;
         }
 
@@ -1187,7 +1187,7 @@ g_checksum_free (GChecksum *checksum)
  * g_checksum_update:
  * @checksum: a #GChecksum
  * @data: buffer used to compute the checksum
- * @length: size of the buffer
+ * @length: size of the buffer, or -1 if it is a null-terminated string.
  *
  * Feeds @data into an existing #GChecksum. The checksum must still be
  * open, that is g_checksum_get_string() or g_checksum_get_digest() must
@@ -1198,11 +1198,13 @@ g_checksum_free (GChecksum *checksum)
 void
 g_checksum_update (GChecksum    *checksum,
                    const guchar *data,
-                   gsize         length)
+                   gssize        length)
 {
   g_return_if_fail (checksum != NULL);
   g_return_if_fail (data != NULL);
-  g_return_if_fail (length > 1);
+
+  if (length < 0)
+    length = strlen (data);
 
   if (checksum->digest_str)
     {
@@ -1281,12 +1283,12 @@ g_checksum_get_string (GChecksum *checksum)
 /**
  * g_checksum_get_digest:
  * @checksum: a #GChecksum
- * @digest: output buffer
+ * @buffer: output buffer
  * @digest_len: an inout parameter. The caller initializes it to the size of @buffer.
  *   After the call it contains the length of the digest.
  *
  * Gets the digest from @checksum as a raw binary vector and places it
- * into @digest. The size of the digest depends on the type of checksum.
+ * into @buffer. The size of the digest depends on the type of checksum.
  *
  * Once this function has been called, the #GChecksum is closed and can
  * no longer be updated with g_checksum_update().
@@ -1388,7 +1390,7 @@ g_compute_checksum_for_data (GChecksumType  checksum_type,
  * g_compute_checksum_for_string:
  * @checksum_type: a #GChecksumType
  * @str: the string to compute the checksum of
- * @length: the length of the string, or -1
+ * @length: the length of the string, or -1 if the string is null-terminated.
  *
  * Computes the checksum of a string.
  *
@@ -1400,7 +1402,7 @@ g_compute_checksum_for_data (GChecksumType  checksum_type,
 gchar *
 g_compute_checksum_for_string (GChecksumType  checksum_type,
                                const gchar   *str,
-                               gsize          length)
+                               gssize         length)
 {
   g_return_val_if_fail (IS_VALID_TYPE (checksum_type), NULL);
   g_return_val_if_fail (str != NULL, NULL);
